﻿using System;
using System.Collections.Generic;
using CeboMsrNet;
using System.IO;

/**
 * CeboMsr - C# example - Info.
 *
 * Print out various information about the device.
 *
 * @author Thomas Hoppe
 */
namespace InfoExample
{
    class InfoExample
    {
        /// <summary>
        /// Print various information to the console.
        /// </summary>
        /// <param name="device">Device to use for test.</param>
        private void PrintInformation(Device device)
        {
            // Put out some device specific information.
            Console.WriteLine("Device type: " + device.DeviceType.Name);
            Console.WriteLine("USB base ver: " + LibraryInterface.UsbBaseVersion);
            Console.WriteLine("API vers: " + LibraryInterface.ApiVersion);
            Console.WriteLine("Firmware ver: " + device.FirmwareVersion);
            Console.WriteLine("Identifier: " + device.Identifier);
            Console.WriteLine("Serial number: " + device.SerialNumber);
            Console.WriteLine("Temperature: " + device.Temperature);

            // Get the real current of the reference current sources.
            foreach (CurrentSource source in device.CurrentSources)
            {
                Console.WriteLine("Reference current of "
                    + source.Name + ": "
                    + source.ReferenceCurrent + " uA");
            }

            // Retrieve information about single ended input #0.
            // This works on all analog inputs and outputs.
            AnalogInput se0 = device.SingleEndedInputs[0];
            Console.WriteLine("Info for single ended input " + se0.Name + ":");
            Console.WriteLine("  Min. ICD: " + se0.MinInterChannelDelay + " us");
            Console.WriteLine("  Current range: "
                + se0.Range.MinValue + " V to "
                + se0.Range.MaxValue + " V");
            Console.WriteLine(  "Supported ranges:");
            for (int i = 0; i < se0.SupportedRanges.Count; ++i)
            {
                Range range = se0.SupportedRanges[i];
                Console.WriteLine("    Range #" + i + " range: "
                    + range.MinValue + " V to "
                    + range.MaxValue + " V, "
                    + "Def. ICD: "
                    + se0.GetDefaultInterChannelDelay(range) + " us");
            }

            // Get info for digital port #0.
            DigitalPort dp0 = device.DigitalPorts[0];
            Console.WriteLine("Info for digital port " + dp0.Name + ":");
            Console.WriteLine("  Count of I/O's: " + dp0.IoCount);
            for (int i = 0; i < dp0.IoCount; ++i)
            {
                Console.WriteLine("    I/O #" + i + ":" + dp0.GetIoName(i));
            }
        }

        /// <summary>
        /// The examples main method.
        /// </summary>
        private void RunExample()
        {
            try
            {
                // Search for devices ...
                IList<Device> devices = LibraryInterface.Enumerate(DeviceType.All);

                // If at least one has been found, use the first one ...
                if (devices.Count > 0)
                {
                    Device device = devices[0];

                    // Open device, nothing can be done without doing this.
                    device.Open();

                    // Put out info.
                    PrintInformation(device);

                    // Finalize device usage, this free's up the device, so it can be used
                    // again, including other applications.
                    device.Close();
                }
            }
            // These exceptions are currently used to report different problems ...
            catch (ArgumentException ex)
            {
                Console.WriteLine(ex.ToString());
            }
            catch (IndexOutOfRangeException ex)
            {
                Console.WriteLine(ex.ToString());
            }
            catch (InvalidOperationException ex)
            {
                Console.WriteLine(ex.ToString());
            }
            catch (IOException ex)
            {
                Console.WriteLine(ex.ToString());
            }
        }

        static void Main(string[] args)
        {
            new InfoExample().RunExample();
        }
    }
}
