﻿// (c) 2013 Cesys GmbH
// Author: Thomas Hoppe
using System;
using System.Collections.Generic;
using System.Text;

namespace CeboMsrNet
{
    /// <summary>
    /// Offers functionality for analog outputs. Direct construction
    /// is not possible. Instances can be retrieved from AnalogOutputs
    /// of the respective Device class.
    /// </summary>
    public sealed class AnalogOutput : IOutput
    {
        private List<Range> _supportedRanges = new List<Range>();
        private Device _device;
        private uint _interfaceId;
        private int _index;
        private string _name;

        internal uint InterfaceId { get { return _interfaceId; } }
        internal int Index { get { return _index; } }

        internal AnalogOutput(Device device, uint interfaceId, int index)
        {
            _device = device;
            _interfaceId = interfaceId;
            _index = index;
            _name = SystemConnector.GetPeripheralName(device.Handle, interfaceId);
            for (int i = 0; ; ++i)
            {
                uint rangeId = SystemConnector.EnumerateRanges(device.Handle, interfaceId, i);
                if (rangeId == SystemConnector.CeboMsrRangeInvalid)
                    break;

                IList<float> minMax = SystemConnector.GetRangeMinMax(device.Handle, interfaceId, rangeId);
                _supportedRanges.Add(new Range(rangeId, minMax[0], minMax[1], 0));
            }
        }

        /// <summary>
        /// List of the supported ranges.
        /// </summary>
        public IList<Range> SupportedRanges { get { return _supportedRanges.AsReadOnly(); } }

        /// <summary>
        /// Set new range.
        /// </summary>
        /// <param name="range"></param>
        public void SetParameters(Range range)
        {
            // No device currently supports this, included for future functionality.
        }

        /// <summary>
        /// Define (identical to SetParameters(Range)) or read active range setting.
        /// </summary>
        public Range Range
        {
            get { return _supportedRanges[0]; }
            set { SetParameters(value); }
        }

        /// <summary>
        /// Set voltage level directly.
        /// </summary>
        public float Value { set { SystemConnector.WriteAnalogOutput(_device.Handle, _interfaceId, value); } }

        /// <summary>
        /// Name of the component.
        /// </summary>
        public string Name { get { return _name; } }

        public override string ToString()
        {
            return "AnalogOutput" + _index;
        }
    }
}
