package com.cesys.cebo.cebomsr;

import java.io.IOException;

/**
 * Offers functionality for triggers. Instances can be retrieved calling getTriggers()
 * of the respective {@link Device} instance.
 *
 * @author Thomas Hoppe
 */
public final class Trigger implements Input {
	/**
	 * The possible trigger configuration values.
	 */
	public enum TriggerConfig {
		OutputPulse(SystemConnector.CeboMsrTriggerOutputPulse),
		OutputAlternating(SystemConnector.CeboMsrTriggerOutputAlternating),
		InputRisingEdge(SystemConnector.CeboMsrTriggerInputRisingEdge),
		InputFallingEdge(SystemConnector.CeboMsrTriggerInputFallingEdge),
		InputAlternating(SystemConnector.CeboMsrTriggerInputAlternating);

		private final int _triggerConfigId;
		private TriggerConfig(int triggerConfigId) {
			_triggerConfigId = triggerConfigId;
		}

		static TriggerConfig getById(int id) {
			switch (id) {
			case SystemConnector.CeboMsrTriggerOutputPulse: return OutputPulse;
			case SystemConnector.CeboMsrTriggerOutputAlternating: return OutputAlternating;
			case SystemConnector.CeboMsrTriggerInputRisingEdge: return InputRisingEdge;
			case SystemConnector.CeboMsrTriggerInputFallingEdge: return InputFallingEdge;
			case SystemConnector.CeboMsrTriggerInputAlternating: return InputAlternating;
			}
			throw new IllegalArgumentException();
		}
	}
	private final Device _device;
	private final int _interfaceId;
	private final int _index;
	private final String _name;

	int getInterfaceId() {
		return _interfaceId;
	}

	int getIndex() {
		return _index;
	}

	Trigger(Device device, int interfaceId, int index) throws IOException {
		_device = device;
		_interfaceId = interfaceId;
		_index = index;
		byte[] tmp = new byte[SystemConnector.MaxIdentLength];
		SystemConnector.checkError(SystemConnector.CeboMsrGetPeripheralName(device.getHandle(), interfaceId, tmp));
		_name = Utils.convertByteArrayToString(tmp);
	}

	/**
	 * Enable or disable trigger.
	 * @param enable State to set.
	 */
	public void setEnabled(boolean enable) throws IOException {
		SystemConnector.checkError(SystemConnector.CeboMsrSetTriggerEnable(_device.getHandle(), getInterfaceId(), enable ? 1 : 0));
	}

	/**
	 * Get current trigger state.
	 */
	public boolean isEnabled() throws IOException {
		int[] r = new int[1];
		SystemConnector.checkError(SystemConnector.CeboMsrGetTriggerEnable(_device.getHandle(), getInterfaceId(), r));
		return r[0] != 0;
	}

	/**
	 * Define behavior of the trigger.
	 * @param triggerConfig Use one of the values from Trigger.TriggerConfig.
	 */
	public void setConfig(TriggerConfig triggerConfig) throws IOException {
		SystemConnector.checkError(SystemConnector.CeboMsrSetTriggerConfig(_device.getHandle(), getInterfaceId(), triggerConfig._triggerConfigId));
	}

	/**
	 * Request configuration of the trigger.
	 * @return One of the values from Trigger.TriggerConfig.
	 */
	public TriggerConfig getConfig() throws IOException {
		int[] r = new int[1];
		SystemConnector.checkError(SystemConnector.CeboMsrGetTriggerConfig(_device.getHandle(), getInterfaceId(), r));
		return TriggerConfig.getById(r[0]);
	}

	/**
	 * Returns name of the component.
	 */
	public String getName() {
		return _name;
	}

	public String toString() {
		return "Trigger" + _index;
	}
}
