package com.cesys.cebo.cebomsr.examples;

import java.io.IOException;
import java.util.List;

import com.cesys.cebo.cebomsr.Device;
import com.cesys.cebo.cebomsr.DeviceType;
import com.cesys.cebo.cebomsr.Input;
import com.cesys.cebo.cebomsr.InputFrame;
import com.cesys.cebo.cebomsr.LibraryInterface;
import com.cesys.cebo.cebomsr.Output;
import com.cesys.cebo.cebomsr.OutputFrame;

/**
 * CeboMsr - Java example - Frame I/O.
 *
 * Multiple inputs are read multiple times in a single transaction,
 * values are printed to the console.
 * AO-1 and DP-2 are set multiple times in a single transaction.
 *
 * Cebo-Stick example does input transfer only.
 *
 * @author Thomas Hoppe
 */
public class FrameIoExample {

	/**
	 * Do frame IO (read/write multiple I/O in single calls).
	 * @param device Device to use for test.
	 * @throws IOException
	 */
	private void frameIoTestLC(Device device) throws IOException {
		// Prepare and fill the list of inputs to read.
		Input[] inputs = new Input[] {
			device.getSingleEndedInputs().get(0),
			device.getSingleEndedInputs().get(1),
			device.getDifferentialInputs().get(1),
			device.getDigitalPorts().get(0),
			device.getDigitalPorts().get(1),
			device.getCounters().get(0)
		};

		// Setup device with this selection.
		device.setupInputFrame(inputs);

		// Read the values multiple times and write them to the console.
		for (int i = 0; i < 100; ++i) {
			// Read all inputs into the instance of InputFrame.
			InputFrame inFrame = device.readFrame();

			// Write results to the console.
			System.out.println(
				"DigitalPort #0: " + inFrame.getDigitalPort(0) + ", " +
				"DigitalPort #1: " + inFrame.getDigitalPort(1) + ", " +
				"SingleEnded #0: " + inFrame.getSingleEnded(0) + " V, " +
				"SingleEnded #1: " + inFrame.getSingleEnded(1) + " V, " +
				"Differential #1: " + inFrame.getDifferential(1) + " V, " +
				"Counter #0: " + inFrame.getCounter(0));
		}

		// Write to analog out #1 and digital out #2 in one call.
		// Prepare and set output selection.
		Output[] outputs = new Output[] {
			device.getDigitalPorts().get(2),
			device.getAnalogOutputs().get(1)
		};

		// Prepare device ...
		device.setupOutputFrame(outputs);

		// Prepare digital port #2 as output.
		device.getDigitalPorts().get(2).setOutputEnableMask(0xf);

		// Create instance of OutputFrame. There's no direct construction, as this
		// instance may vary between different device types.
		OutputFrame outFrame = device.createOutputFrame();

		// Write it to hardware, modify contents in every looping.
		for (int i = 0; i < 100; ++i) {
			outFrame.setAnalogOutput(1, (float)(3 * Math.sin((float)i / 100.f)));
			outFrame.setDigitalPort(2, i);

			device.writeFrame(outFrame);
		}

		// Reset to power up defaults.
		device.resetDevice();
	}

	/**
	 * Do frame IO (read/write multiple I/O in single calls).
	 * @param device Device to use for test.
	 * @throws IOException
	 */
	private void frameIoTestStick(Device device) throws IOException {
		// Prepare and fill the list of inputs to read.
		Input[] inputs = new Input[] {
			device.getSingleEndedInputs().get(0),
			device.getSingleEndedInputs().get(1),
			device.getDigitalPorts().get(0),
			device.getCounters().get(0)
		};

		// Setup device with this selection.
		device.setupInputFrame(inputs);

		// Read the values multiple times and write them to the console.
		for (int i = 0; i < 100; ++i) {
			// Read all inputs into the instance of InputFrame.
			InputFrame inFrame = device.readFrame();

			// Write results to the console.
			System.out.println(
				"DigitalPort #0: " + inFrame.getDigitalPort(0) + ", " +
				"SingleEnded #0: " + inFrame.getSingleEnded(0) + " V, " +
				"SingleEnded #1: " + inFrame.getSingleEnded(1) + " V, " +
				"Counter #0: " + inFrame.getCounter(0));
		}

		// Reset to power up defaults.
		device.resetDevice();
	}

	/**
	 * The examples main method.
	 */
	private void runExample() {
		try {
			// Search for devices ...
			List<Device> devices = LibraryInterface.enumerate(DeviceType.All);

			// If at least one has been found, use the first one ...
			if (!devices.isEmpty()) {
				Device device = devices.get(0);

				// Open device, nothing can be done without doing this.
				device.open();

				// *** DIGITAL PORTS AND ANALOG OUTPUTS ARE MODIFIED DURING THIS ***
				// *** SECTION, SO DO NOT CONNECT ANY PERIPHERALS WITHOUT STUDYING ***
				// *** THE ACTIONS DONE HERE, OTHERWISE YOUR PERIPHERALS MAY BE DAMAGED !!! ***
				if (device.getDeviceType() == DeviceType.CeboLC)
					frameIoTestLC(device);
				else if (device.getDeviceType() == DeviceType.CeboStick)
					frameIoTestStick(device);
				else
					throw new RuntimeException("No supported device detected.");

				// Finalize device usage, this free's up the device, so it can be used
				// again, including other applications.
				device.close();
			}
		}
		catch (IOException ex) {
			ex.printStackTrace();
		}
	}

	public static void main(String[] args) {
		new FrameIoExample().runExample();
	}
}
